@@ -52,8 +52,8 @@ logging.basicConfig(filename="deepa2.log", level=logging.DEBUG)
 app = typer.Typer()
 
 
-@app.command()
-def bake(  # pylint: disable=too-many-arguments,too-many-branches,too-many-statements # noqa: C901
+@app.command()  # noqa: C901
+def bake(  # noqa: C901  pylint: disable=too-many-arguments,too-many-branches,too-many-statements
     source_type: Optional[str] = typer.Option(
         None,
         help="type of the source dataset, used to"
