@@ -6,6 +6,7 @@ import re
 from typing import Any, List, Dict, Tuple, Optional, Union
 
 import jinja2
+from nltk.sem.logic import Expression, LogicalExpressionException  # type: ignore
 from ttp import ttp  # type: ignore
 
 from deepa2 import DeepA2Item, QuotedStatement, ArgdownStatement, Formalization
@@ -63,7 +64,19 @@ class DeepA2Layouter:  # pylint: disable=too-few-public-methods
             List[Tuple[str, str]],
             Union[List[Tuple[str, str]], None],
         ]:
-            return self._format_dict(dict(data))
+            try:
+                dictdata = dict(data)
+            except Exception as exc:
+                logging.error(
+                    "DeepA2Layouter couldn't format data %s in field %s as dict",
+                    data,
+                    field,
+                )
+                logging.error(exc)
+                raise ValueError(
+                    f"DeepA2Layouter couldn't format data {data} in field {field} as dict"
+                ) from exc
+            return self._format_dict(dictdata)
 
         if field.type in [
             Union[List[QuotedStatement], None],
@@ -164,6 +177,13 @@ class DeepA2Parser:
     def parse_list(text: str):
         """parses list of statements"""
 
+    @staticmethod
+    def parse_quotes(text: str) -> Optional[List[Optional[QuotedStatement]]]:
+        """parses formalizations"""
+        parser = QuotationsParser()
+        quotes = parser.parse_quotes(text)
+        return quotes
+
     @staticmethod
     def parse_formalization(text: str) -> Optional[List[Optional[Formalization]]]:
         """parses formalizations"""
@@ -172,8 +192,190 @@ class DeepA2Parser:
         return formalizations
 
     @staticmethod
-    def parse_keys(text: str):
+    def parse_keys(text: str) -> Optional[List[Optional[Tuple[str, str]]]]:
         """parses keys of formalization"""
+        parser = KeysParser()
+        keys = parser.parse_keys(text)
+        return keys
+
+    @staticmethod
+    def parse_as_folf(
+        formalizations: List[Optional[Formalization]] = None,
+    ) -> Optional[List[Any]]:
+        """parses formalizations as first-order-logic formulae"""
+        parser = FOLParser()
+        folfs = parser.parse(formalizations)
+        return folfs
+
+
+class FOLParser:
+    """parser methods for first-order-logic formulae
+    based on NLTK parser"""
+
+    @staticmethod
+    def parse(
+        formalizations: List[Optional[Formalization]] = None,
+    ) -> Optional[List[Optional[Expression]]]:
+        """parses da2 formalizations as NLTK first-order-logic formulae"""
+        # preprocess
+        preprocessed_forms = FOLParser._preprocess(formalizations)
+        print(preprocessed_forms)
+        expressions = []
+        for pp_form in preprocessed_forms:
+            try:
+                expressions.append(Expression.fromstring(pp_form))
+            except LogicalExpressionException:
+                expressions.append(None)
+        return expressions
+
+    @staticmethod
+    def _preprocess(formalizations: List[Optional[Formalization]] = None) -> List[str]:
+        """preprocesses formalizations"""
+        if not formalizations:
+            return []
+        preprocessed_forms = []
+        for formalization in formalizations:
+            if formalization:
+                formula = FOLParser._da2_to_nltk_format(formalization.form)
+                preprocessed_forms.append(formula)
+            else:
+                preprocessed_forms.append("")
+        return preprocessed_forms
+
+    @staticmethod
+    def _da2_to_nltk_format(form: str) -> str:
+        """converts DeepA2 formalization to NLTK format"""
+        formula = form
+
+        # 1. reformat quantifiers
+        # find all subsequences with quantifiers
+        regex = r"(?:\s?\([E]?.\)\s?)+:"
+        matches = re.finditer(regex, formula, re.MULTILINE)
+        tmp = ""
+        pointer = 0
+        count_brackets_to_close = 0
+        for match in matches:
+            tmp += formula[pointer : match.start()]
+            tmp += FOLParser.replace_quantifiers(match.group())
+            # if formula doesn't continue with opening bracket, add one,
+            # assuming that the remainder of the formula belongs to scope of
+            # these quantors
+            if not formula[match.end() :].strip().startswith("("):
+                tmp += "("
+                count_brackets_to_close += 1
+            pointer = match.end()
+        tmp += formula[pointer:]
+        tmp += ")" * count_brackets_to_close
+        formula = tmp
+
+        # 2. reformat predicative subsentences
+        # find all atomic predicative formulas
+        regex = r"[A-Z](\s[a-uw-z])+(?:\s|\)|$)"
+        matches = re.finditer(regex, formula, re.MULTILINE)
+        tmp = ""
+        pointer = 0
+        for match in matches:
+            tmp += formula[pointer : match.start()]
+            tmp += FOLParser.format_predicative_f(match.group())
+            pointer = match.end()
+        tmp += formula[pointer:]
+        formula = tmp
+
+        # 3. replace junctors
+        formula = formula.replace("v", "|")
+        formula = formula.replace("not", "!")
+
+        return formula
+
+    @staticmethod
+    def replace_quantifiers(quantifiers: str):
+        """Replaces individual quantifiers in a sequence of quantifiers"""
+        new_s = ""
+        # find all individual quantifiers
+        regex = r"\(([E]?)(.)\)"
+        matches = re.finditer(regex, quantifiers, re.MULTILINE)
+        for match in matches:
+            quantifier = "exists" if bool(match.group(1)) else "all"
+            variable = match.group(2)
+            new_s += f"{quantifier} {variable}."
+
+        return new_s
+
+    @staticmethod
+    def format_predicative_f(predicative_f: str):
+        """formats predicative formula, e.g. 'A x y' to 'A(x,y)'"""
+        predicate = predicative_f[0]
+        arguments = predicative_f[1:]
+        arguments = arguments.strip()
+        arguments_list = arguments.split(" ")
+        arguments = ",".join(arguments_list)
+        new_f = f"{predicate}({arguments})"
+        return new_f
+
+
+class QuotationsParser:
+    """parses text as list of quotes"""
+
+    @staticmethod
+    def parse_quotes(text: str) -> Optional[List[Optional[QuotedStatement]]]:
+        """tries to parse text as list of Formalizations"""
+        if text is None:
+            return None
+        if not text:
+            return []
+        sep = DeepA2Layouter.list_seperator()
+        qtexts = text.split(sep)
+        quotes = []
+        for qtext in qtexts:
+            quotes.append(QuotationsParser.parse_quotetext(qtext))
+        return quotes
+
+    @staticmethod
+    def parse_quotetext(text: str) -> Optional[QuotedStatement]:
+        """tries to parse text as a single quote"""
+
+        template = "{{ text  | ORPHRASE }} (ref: ({{ ref_reco }}))"
+        text = text.strip()
+
+        parser = ttp(text, template)
+        parser.parse()
+        parse_results = parser.result()[0][0]
+        if not ("text" in parse_results and "ref_reco" in parse_results):
+            return None
+        try:
+            ref_reco = int(parse_results["ref_reco"])
+        except ValueError:
+            ref_reco = -1
+        quote = QuotedStatement(text=parse_results["text"], ref_reco=ref_reco)
+        return quote
+
+
+class KeysParser:
+    """parses text as list of keys"""
+
+    @staticmethod
+    def parse_keys(text: str) -> Optional[List[Optional[Tuple[str, str]]]]:
+        """tries to parse text as list of keys"""
+        if text is None:
+            return None
+        if not text:
+            return []
+        sep = DeepA2Layouter.list_seperator()
+        ktexts = text.split(sep)
+        keys = []
+        for ktext in ktexts:
+            keys.append(KeysParser.parse_key(ktext))
+        return keys
+
+    @staticmethod
+    def parse_key(text: str) -> Optional[Tuple[str, str]]:
+        """tries to parse text as a key:value pair"""
+        if ":" not in text:
+            return None
+        text = text.strip()
+        # split at first colon
+        key, value = text.split(":", 1)
+        return key.strip(), value.strip()
 
 
 class FormulaeParser:
